/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
***********************************************************************/

import java.awt.*;

/**
 * Classe astratta le cue estensioni specificano come deve avvenire  l'istanziazione dei componenti dell'interfaccia
 * per l'applicazione.
 *
 * @see METAComponentFactory
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto
 */
public abstract class ComponentFactory
{
	/**
	 * Crea un nuovo componente che rispetta le specifiche dell'interfaccia IWindow.
	 * @see IWindow
	 * @see METAWindow
	 */
    public abstract IWindow createWindow(String title, Dimension size);

	/**
	 * Crea un nuovo componente che rispetta le specifiche dell'interfaccoa IContainer.
	 * @see IContainer
	 * @see METAContainer
	 */
    public abstract IContainer createContainer(Dimension size);

	/**
	 * Crea un nuovo componente che rispetta le specifiche dell'interfaccia ILabel.
	 * @see ILabel
	 * @see METALabel
	 */
    public abstract ILabel createLabel(String label, Dimension size);

	/**
	 * Crea un nuovo componente che rispetta le specifiche dell'interfaccia IButton.
	 * @see IButton
	 * @see METAButton
	 */
    public abstract IButton createButton(String label, Dimension size);

	/**
	 * Crea un nuovo componente che rispetta le specifiche dell'interfaccia IText.
	 * @see IText
	 * @see METAText
	 */
    public abstract IText createText(String label, Dimension size);

	/**
	 * Crea un nuovo componente che rispetta le specifiche dell'interfaccia IBooleanButton.
	 * @see IBooleanButton
	 * @see METABooleanButton
	 */
    public abstract IBooleanButton createBooleanButton(boolean value, Dimension size);
}

