/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  
USA
***********************************************************************/

import java.util.*;
import java.awt.*;

/**
 * <table align=center>
 * 		<tr>
 * 			<td align=center><img src=dogecalc.jpg></td>
 * 			<td align=center bgcolor="#c0c0c0" width=183><h3>Dogecalc</h3></td>
 * 		</tr>
 * <table>
 * 
 * Semplice implementazione di una calcolatrice utilizzabile attraverso tastiera e mouse, in grado di effettuare le quattro
 * operazioni fondamentali, due funzioni trigonometriche, una delle composizioni delle due il tutto anche sotto
 * forma di espressioni miste.
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto
 *
 * <table align=center>
 * 		<tr>
 * 			<td align=center><p>MeTA~LAbS >> building information subways...</p><td>
 * 			<td align=center><img src=blocco_release.jpg></td>
 * 		</tr>
 * </table>
 */
public class Dogecalc
{
    /**
	 * Si occupa di organizzare tutti gli operatori presenti nella stringa acquisita da <code>METAText</code>
	 * per inserirli in una struttura di tipo <code>ArrayList</code>, per fornire una espressione
	 * facilmente calcolabile dalla metodo <code>computeArray</code>.
     */
    public static float computeString(String s)
    {
	int i ;
	String tag = ""; // campo che memorizza gli operatori oggettizzati dalla stringa
	char ch, first=s.charAt(0); // carattere di controllo per eventuali malformazioni della stringa in ingresso
	ArrayList a = new ArrayList(); // array costipativo contenente gli elementi suddivisi
	
	if((Character.isDigit(first)  ||  first == '_'  ||  first == 'S'  ||  first == 'C'  ||  first == 'T')  &&  Character.isDigit(s.charAt(s.length() - 1)))
	    {
		for(i = 0; i < s.length(); i ++)
		    {
			ch = s.charAt(i);
			switch(ch)
			    {
				// controllo su seni/coseni/tangenti
			    case 'S':
				a.add(new String("sin"));
				i += 2;
				break;
			    case 'C':
				a.add(new String("cos"));
				i += 2;
				break;
			    case 'T':
				a.add(new String("tan"));
				i += 2;
				break;
				// controllo su numeri negativi
			    case '_':
				if(tag.length() == 0)
				    tag = new String(tag + '-');
				else return Float.NaN;
				break;
				// in tutti gli altri casi, gestione di numeri, virgole ecc...
			    default:    
				if(Character.isDigit(ch))
				    tag = new String(tag + ch);
				else if(ch == '.'  &&  tag.indexOf('.') < 0)
				    tag = new String(tag + '.');
				else
				    {
					if(tag.length() > 0  &&  ((Character.isDigit(tag.charAt(0)) || (tag.charAt(0) == '-'  &&  Character.isDigit(tag.charAt(1)))) && (Character.isDigit(tag.charAt(tag.length() - 1)))))
					    {
						a.add(tag);
						tag = new String("");
						a.add(new String("" + ch));
					    }
					else return Float.NaN;
				    }
			    }
		    }
	    }
	else return Float.NaN;
	
	a.add(tag);
	return computeArray(trigoComputeArray(a));
    }

    /**
	 * Funzione che riduce le funzioni trigonometriche a valori reali all'interno dell'espressione rappresentata
	 * in un <code>ArrayList</code> ottenuta a seguito del primo passaggio del flusso ottenuto da <code>METAText</code>
	 * attraverso il metodo <code>computeString</code>.
     */
    public static ArrayList trigoComputeArray(ArrayList a)
    {
	ArrayList baz = new ArrayList();
	Iterator i = a.iterator();
	
	try
	    {
		while(i.hasNext())
		    {
			String next = (String)i.next();
			// calcolo di seni/coseni/tangenti sostituiti dai valori numerici
			if(next.equals("sin"))
			    baz.add(String.valueOf(Math.sin(Math.toRadians(new Double((String)i.next()).doubleValue()))));
			else if(next.equals("cos"))
			    baz.add(String.valueOf(Math.cos(Math.toRadians(new Double((String)i.next()).doubleValue()))));
			else if(next.equals("tan"))
			    baz.add(String.valueOf(Math.tan(Math.toRadians(new Double((String)i.next()).doubleValue()))));
			else
			    baz.add(next);
		    }
		return baz;
	    } catch (NumberFormatException e)
		{
		    // gestione delle eccezioni per quanto riguarda l'overflow dovuto al campo di esistenza delle funzioni trigonometriche
			// o alla malformazione dell'espressione introdotta dell'utente.
		    baz = null;
		    System.out.println("Exception occurred on trigoComputeArray(ArrayList a): NumberFormatException");
		    return baz;
		}
    }

    /**
	 * Restituisce il risultato dell'espressione contenuta nell'<code>ArrayList</code> rispettando le dovute regole
	 * di priorit tra le operazioni.
     */
    public static float computeArray(ArrayList a)
    {
	float operand = 0;
	String next;
	
	if(a == null)
	    return Float.NaN;
	
	try
	    {
		while(a.size() > 0)
		    {
			next = (String)a.get(0);
			// operatore somma: aggiungo l'addendo al risultato ricorsivo del resto dell'array
			if(next.equals("+"))
			    {
				a.remove(0);
				operand = operand + computeArray(a);
				return operand;
			    }
			// operatore sottrazione: viene calcolato in base all'operazione successiva tenendo
			// conto della priorit che l'operazione necessita
			else if(next.equals("-")) 
			    {
				a.remove(0);
				if(a.size() > 1) 
				    next = (String)a.get(1);
				else 
				    {
					operand = operand - Float.parseFloat((String)a.get(0));
					a.remove(0);
					return operand;
				    }
				if(next.equals("-") || next.equals("+"))
				    {
					operand = operand - Float.parseFloat((String)a.get(0));
					a.remove(0);
					operand = operand + computeArray(a);
				    }
				else operand = operand - computeArray(a);
			    }
			// operazioni moltiplicazione e divisione, vengono computate istantaneamente 
			// sostituendo il risultato all'operando attuale
			else if(next.equals("*"))
			    {
				a.remove(0);
				operand = operand * Float.parseFloat((String)a.get(0));
				a.remove(0);
			    }
			else if(next.equals("/"))
			    {
				a.remove(0);
				operand = operand / Float.parseFloat((String)a.get(0));
				a.remove(0);
			    }
			else
			    {
				operand = Float.parseFloat(next);
				a.remove(0);
			    }
		    }
	    } catch (NumberFormatException e)
		{
		    // gestione di eccezione del formato di numero con indicazione di tale occorrenza
		    System.out.println("Exception occurred on ComputeArray(ArrayList a): NumberFormatException");
		    return Float.NaN;
		}
	
	return operand;
    }
    
	/**
	 * Metodo principale dell'applicazione: la sua chiamata comporta l'inizio del tempo di esecuzione
	 * della calcolatrice, ovvero l'inizio della possibilit di calcolare. Tutto questo pu diventare realt
	 * invocando l'interprete di bytecode Java con la classe Dogecalc.
	 */
    public static void main(String[] args)
    {
	String TITLE = "DogeCALC               - MeTALAbS - ";

	ComponentFactory cf = new METAComponentFactory();
	// istanziazione della finestra principale dell'applicazione, del container e delle aree di testi che rappresentano
	// il display di visualizzazione ed editing della calcolatrice.
	final IWindow calculator = cf.createWindow(TITLE, new Dimension(280, 400));
	final IContainer container = cf.createContainer(new Dimension(600, 400));
	final ILabel display = cf.createLabel(null, new Dimension(250, 30));
	final IText  insert = cf.createText(null, new Dimension(250, 30));
	
	// istanziazione di tutti i pulsanti della coalcolatrice.
	final IButton zero = cf.createButton("0", new Dimension(40, 30));
	final IButton uno = cf.createButton("1", new Dimension(40, 30));
	final IButton due = cf.createButton("2", new Dimension(40, 30));
	final IButton tre = cf.createButton("3", new Dimension(40, 30));
	final IButton quattro = cf.createButton("4", new Dimension(40, 30));
	final IButton cinque = cf.createButton("5", new Dimension(40, 30));
	final IButton sei = cf.createButton("6", new Dimension(40, 30));
	final IButton sette = cf.createButton("7", new Dimension(40, 30));
	final IButton otto = cf.createButton("8", new Dimension(40, 30));
	final IButton nove = cf.createButton("9", new Dimension(40, 30));	
	final IButton som = cf.createButton("+", new Dimension(40, 30));
	final IButton sott = cf.createButton("-", new Dimension(40, 30));
	final IButton molt = cf.createButton("*", new Dimension(40, 30));
	final IButton div = cf.createButton("/", new Dimension(40, 30));
	final IButton cl = cf.createButton("clear", new Dimension(90, 30));
	final IButton punto = cf.createButton(".", new Dimension(40, 30));
	final IButton uguale = cf.createButton("=", new Dimension(40, 30));
	final IButton del = cf.createButton("<-", new Dimension(40, 30));
	final IButton neg = cf.createButton("-/+", new Dimension(40, 30));
	final IButton esc = cf.createButton("Esc", new Dimension(90, 30));
	final IButton sin = cf.createButton("Sin", new Dimension(40, 30));	
	final IButton cos = cf.createButton("Cos", new Dimension(40, 30));	
	final IButton tan = cf.createButton("Tan", new Dimension(40, 30));
	final IButton help = cf.createButton("Help", new Dimension(40, 30));
	final IBooleanButton bb = cf.createBooleanButton(false, new Dimension(20, 20));
	
	// l'oggetto display si limita cancellare il suo contenuto nel caso che si invii il comando C (cancel)
	// oppure rappresentare il carattere passato in tutti gli altri casi.
	display.setListener(new METAListener()
	    {
		public boolean handle(IComponent c, IEvent e)
		{
		    if(e.getClass().getName().equals("METAKeyReleasedEvent"))
			{
			    char ch =  ((METAKeyReleasedEvent)e).getChar();
			    
			    if(ch == 'C')
				((METALabel)c).setLabel("");
			    else
				{
				    ((METALabel)c).setLabel(((METALabel)c).getLabel() + ch);
				    return true;
				}
			    return false;
			}
		    else return false;
		}
	    });

	// gli oggetti sin, cos e tan mandano al display i tre caratteri consecutivi delle loro iniziali (bube)
	sin.setListener(new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    int i;
		    for(i = 0; i < 3; i++)
			insert.handle(new METAKeyReleasedEvent(sin.getLabel().charAt(i)));
		    return true;
 		}
	    });

	cos.setListener(new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    int i;
		    for(i = 0; i < 3; i++)
			insert.handle(new METAKeyReleasedEvent(cos.getLabel().charAt(i)));
		    return true;
 		}
	    });

	tan.setListener(new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    int i;
		    for(i = 0; i < 3; i++)
			insert.handle(new METAKeyReleasedEvent(tan.getLabel().charAt(i)));
		    return true;
 		}
	    });

	// i valori negativi per questioni di praticit nella rappresentazioni sono prefissi dal carattere speciale "_" underscore.
	neg.setListener(new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    insert.handle(new METAKeyReleasedEvent('_'));
		    return true;
 		}
	    });

	esc.setListener(new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    insert.handle(new METAKeyReleasedEvent('q'));
		    return true;
 		}
	    });

	del.setListener(new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    insert.handle(new METAKeyReleasedEvent('d'));
		    return true;
 		}
	    });

	uguale.setListener(new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    insert.handle(new METAKeyReleasedEvent('u'));
		    return true;
 		}
	    });
	
	// il pulsante booleano si limita a cambiare di stato ad ogni pressione del mouse.
	bb.setListener(new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    if(bb.getValue())
			{
		    	bb.setValue(false);
			return true;
			}
		    else 
			{
			    bb.setValue(true);
			    return true;
			}
 		}
	    });
	
	// gestione di IText
  	insert.setListener(new METAListener()
  	    {
  		public boolean handle(IComponent c, IEvent e)
  		{
  		    String text = ((METAText)c).getText();
  		    char ch = ((METAKeyReleasedEvent)e).getChar();
		    
		    // cancella
 		    if(ch == 'c')
 			((METAText)c).setText("");
		    // quitta
 		    else if(ch == 'q')
 			System.exit(0);
    		    else if(text != null  &&  text.length() > 0)
 			{
			    // cancella un carattere
 			    if(ch == 'd')
 				((METAText)c).setText(text.substring(0, text.length() -1));
			    // uguale, calcola il risultato
			    else if(ch == 'u')
 				{
 				    int foo;
				    String strComp;
 				    float f = computeString(((METAText)c).getText());

				    if(f == Float.NaN)
					strComp = "NaN";
				    else
					strComp = String.valueOf(f);
				    display.handle(new METAKeyReleasedEvent('C'));
 				    for(foo = 0; foo < strComp.length(); foo ++)
 					display.handle(new METAKeyReleasedEvent(strComp.charAt(foo)));
 				}
			    // altrimenti aggiungi semplicemente in coda il carattere
 			    else ((METAText)c).setText(text + ch);
 			}
 		    else if(ch != 'd'  &&  ch != 'u') ((METAText)c).setText("" + ch);
 		    return true;
 		}
  	    });

	// gestione del pulsante  di aiuto sovrastrutturale del Doge in persona del quale lo ringraziamo
	// ogni giorno attraverso l'olocausto dell'incendio di panette di burro e pietre miliari
	help.setListener(new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    Random r = new Random();
		    int i, n = r.nextInt(7);
		    String doge = "";
		    
		    switch(n)
			{
			case 0:
			    doge = "Il Doge ti considera";
			    break;
			case 1:
			    doge = "Sii un uomo specifico del foo";
			    break;
			case 2:
			    doge = "Il foo e' la primitiva del bube stesso";
			    break;
			case 3:
			    doge = "Il Doge e' sempre presso te stesso";
			    break;
			case 4:
			    doge = "Il Destraminio";
			    break;
			case 5:
				doge = "Il Sacripante sostiene la tua causa";
				break;
			case 6:
				doge = "Non temi la pozzanghera di velcro";
				break;
			}

		    display.handle(new METAKeyReleasedEvent('C'));
		    
		    for(i = 0; i < doge.length(); i ++)
			display.handle(new METAKeyReleasedEvent(doge.charAt(i)));
		    
		    return true;
 		}
	    });

	// gestore di eventi generico per pulsanti generici
	METAListener buttonListener = new METAListener() 
 	    {
 		public boolean handle(IComponent c, IEvent e)
 		{
		    insert.handle(e);
		    return true;
 		}
	    };

	display.setLabel("0");
	
	// setting di un po' di hotkey dei pulsanti
	((METAButton)del).setHotKey('d');
	((METAButton)neg).setHotKey('_');
	((METAButton)uguale).setHotKey('u');
	((METAButton)esc).setHotKey('q');
	((METAButton)help).setHotKey('h');
	
	// assegna ad ogni pultante il listener sulla tastiera: ogni componente consumer l'evento se sar predisposto a farlo
	zero.setListener(buttonListener);
	uno.setListener(buttonListener);
	due.setListener(buttonListener);
	tre.setListener(buttonListener);
	quattro.setListener(buttonListener);
	cinque.setListener(buttonListener);
	sei.setListener(buttonListener);
	sette.setListener(buttonListener);
	otto.setListener(buttonListener);
	nove.setListener(buttonListener);
	som.setListener(buttonListener);
	sott.setListener(buttonListener);
	molt.setListener(buttonListener);
	div.setListener(buttonListener);
	punto.setListener(buttonListener);
	cl.setListener(buttonListener);

	// aggiunge al container associato alla finestra tutti i sotto-componenti necessari
  	container.add(10, 10, display);
    container.add(10, 50, insert);
   	container.add(10, 330, zero);
   	container.add(10, 290, uno);
   	container.add(60, 290, due);
  	container.add(110, 290, tre);
   	container.add(10, 250, quattro);
   	container.add(60, 250, cinque);
  	container.add(110, 250, sei);
   	container.add(160, 290, som);
   	container.add(210, 290, sott);
  	container.add(160, 250, molt);
   	container.add(210, 250, div);
   	container.add(160, 170, esc);
  	container.add(230, 90, bb);
   	container.add(60, 330, punto);
 	container.add(110, 330, neg);
   	container.add(210, 330, del);
   	container.add(160, 330, uguale);
 	container.add(10, 90, help);
 	container.add(10, 210, sette);
 	container.add(60, 210, otto);
 	container.add(110, 210, nove);
	container.add(110, 170, tan);
	container.add(10, 170, sin);
	container.add(60, 170, cos);
	container.add(160, 210, cl);
	
	// impostra il container associato alla finestra
	calculator.setContent(container);

	// avvia il processo di gestione visuale dell'applicazione
	((METAWindow)calculator).show();
    }
}

