/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
***********************************************************************/

import java.awt.*;

/**
 * E' l'interfaccia base per tutti i sotto-componenti che la estendono all'interno di un <code>IContainer</code>.
 */
public interface IComponent
{
	/** 
	 * Disegna il componente sullo schermo e tutti i suoi sotto-componenti se il componente  un contenitore.
	 * @see IContainer
	 */
	public void paint(java.awt.Graphics g);

	/** 
	 * L'implementazione consente di verificare se il componente  in grado di gestire l'evento
	 * passato come parametro e se possibile lo consuma; se il componente  un contenitore dovr
	 * occurarsi di propagare l'evento a tutti i suoi sotto-componenti.
	 */
    public boolean handle(IEvent event);

	/** 
	 * L'implementazione permette di associare un Listener al componente.
	 * @see IListener
	 */
    public void setListener(IListener listener);

	/** L'implementazione restituisce il riferimento al listener associato al componente. */
    public IListener getListener();

	/** L'implementazione restituisce le dimensioni del componente */
    public Dimension getSize();

	/** 
	 * L'implementazione restituisce il sotto-componente (nel caso di uno o pi componenti contenitori annidati)
	 * pi profondo che contiene sulla sua superficie le coordinate specificate.
	 */
    public IComponent getComponent(int x, int y);
}

