/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
***********************************************************************/

import java.util.*;
import java.awt.*;

/**
 * Un oggetto appartenente alla classe <code>METABooleanButton</code>  un bottone che pu
 * assumere i due stati checked e unchecked. Il bottone passa da uno stato all'altro ad ogni
 * pressione del mouse.
 *
 * @see METAMousePressedEvent
 * @see METAMouseReleasedEvent
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto
 */
public class METABooleanButton extends METAComponent implements IBooleanButton
{
	/** Campo che contiene lo stato attuale del bottone. */
    private boolean value;
	
	/** Colore di sfondo del bottone. */
    private Color bgColor;
	
	/** Colore del bottone quando si trova nello stato checked (premuto). */
    private Color fgColor;
	
	/** 
	 * Crea un nuovo bottone di tipo <code>METABooleanButton</code> con le caratteristiche iniziali
	 * specificate attraverso i parametri.
	 * 
	 * @param w larghezza del bottone
	 * @param h altezza del bottone
	 * @param v stato iniziale del bottone (false = unchecked, true = checked)
	 * @param bg colore di sfondo del bottone
	 * @param fg colore del bottone quando premuto (value = true)
	 */
    public METABooleanButton(int w, int h, boolean v, Color bg, Color fg)
    {
	this.width = w;
	this.height = h;
	value = v;
	bgColor = bg;
	fgColor = fg;
    }
	
	/** Restituisce lo stato attuale del bottone. */
    public boolean getValue()
    {
	return value;
    }
	
	/** Altera lo stato del bottone aggiornandone al contempo l'aspetto grafico. */
    public void  setValue(boolean v)
    {
	value = v;
	this.paint(this.graphics);
    }
	
	/** 
	 * Si occupa di disegnare il bottone nell'oggetto <code>Graphics</code> specificato.
	 * 
	 * @param g oggetto graphics in cui disegnare il bottone
	 */
    public void paint(Graphics g)
    {
	this.graphics = g;
	g.setColor(Color.black);
	g.drawOval(0, 0, this.width, this.height);
	g.setColor(bgColor);
	g.fillOval(0, 0, this.width, this.height);
	g.setColor(fgColor);
	if(value)
	    g.fillOval(this.width * 20/100, this.height * 20/100, this.width * 60/100, this.height * 60/100);
    }
	
	/**
	 * Permette di verificare se il componente  in grado di gestire l'evento passato e restituisce true se l'evento  stato
	 * consumato, false in caso contrario.
	 */
    public boolean handle(IEvent e)
    {
	if(e.getClass().getName().equals("METAMouseReleasedEvent")) 
	    {
		if(this.getListener() != null) 
		    {
			this.getListener().handle(this, e);
			paint(this.graphics);
			return true;
		    }
		else return false;
	    }
	else return false;
    }
}

