/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
***********************************************************************/

import java.util.*;
import java.awt.*;

/**
 * <code>METAComponent</code>  una classe astratta che implementa parzialmente i metodi comuni a tutti i
 * sotto-componenti appartenenti all'interfaccia <code>IComponent</code>.
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto.
 */
public abstract class METAComponent implements IComponent
{	
	/** Altezza del rettalgolo che rappresenta il componente. */
    protected int height;

	/** Larghezza del rettangolo che rappresenta il componente. */
    protected int width;

	/** 
	 * Campo riferimento all'oggetto 
	 * @see METAListener .
	 */
    protected IListener listener = null;

	/** Riferimento all'oggetto <code>Graphics</code> associato al componente. */
    protected Graphics graphics;

	/** Restituisce il riferimento al componente che comprende nella sua superficie le coordinate specificate. */
    public IComponent getComponent(int x, int y)
    {
	return this;
    }
	
	/** 
	 * Riturona il riferimento all'oggetto Listener associato al componente.
	 * @see METAListener
	 */
    public IListener getListener()
    {
	return listener;
    }
	
	/**
	 * Associa al componente l'oggetto Listener specificato come parametro.
	 * @see METAListener
	 */
    public void setListener(IListener i)
    {
	listener = i;
    }
	
	/** Restituisce la dimensione del componente. */
    public Dimension getSize()
    {
	return new Dimension(width, height);
    }
	
	/** 
	 * Metodo astratto che ha la funzione di indicare la necessit di implementare per ogni classe estesa il metodo
	 * necessaio a stabilire se degli eventi devono essere consumati o meno dal componente.
	 */
    public abstract boolean handle(IEvent e);

	/**
	 * Metodo astratto che ha la funzione di indicare la necessit di implementare per ogni classe estesa il metodo
	 * necessario per gestire il ridisegno del componente all'interno dell'applicazione.
	 */
    public abstract void paint(Graphics g);
}

