/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
***********************************************************************/

import java.util.*;
import java.awt.*;

/**
 * Estende la classe <code>ComponentFactory</code> per adeguare la creazione di componenti secondo le regole
 * stabilite dalla classe astratta <code>ComponentFactory</code>.
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto
 */
public class METAComponentFactory extends ComponentFactory
{
	/** Colore di sfondo di default per <code>METAWindow</code>. */
    protected Color bgColor = new Color(120, 120, 255);

	/** Colore di sfondo di default per i bottoni. */
    protected Color  buttonColor = Color.blue;

	/** Colore di default per il testo contenuto nei bottoni. */
    protected Color buttonTextColor = Color.white;

	/** Colore di default per il testo di <code>METAText</code> e <code>METALabel</code>. */
    protected Color textColor = Color.black;
	
	/** Colore di default per lo sfondo di <code>METAText</code>. */
    protected Color textbgColor = Color.white;

	/** Colore di default per lo sfondo di <code>METALabel</code>. */
    protected Color  labelbgColor = Color.white;

	/** Font di default per tutti i componenti che fanno uso di testo. */
    protected Font textFont = Font.decode("Arial-BOLD-12");;

	/** Stato iniziale di default per i componenti <code>METABooleanButton</code>. */
    protected boolean booleanbuttonValue = false;

	
	/**
	 * Restituisce il riferimento ad un nuovo componente di tipo <code>METAWindow</code> con titolo e dimensioni specificate.
	 *
	 * @param s stringa sulla barra del titolo della finestra
	 * @param d dimensioni della finestra
	 */
    public IWindow createWindow(String s, Dimension d)
    {
	return new METAWindow(s, bgColor, d);
    }
	
	/** Restituisce il riferimento ad un nuovo componente di tipo <code>METAContainer</code> delle dimensioni specificate. */
    public IContainer createContainer(Dimension d)
    {
	return new METAContainer();
    }
	
	/**
	 * Restituisce il riferimento ad un nuovo componente di tipo <code>METAButton</code> contenente la stringa spacificata
	 * e delle dimensioni indicate.
	 */
    public IButton createButton(String s, Dimension d)
    {
	return new METAButton(d.width, d.height, s, buttonColor, buttonTextColor, textFont, s.charAt(0));
    }
	
	/**
	 * Restituisce il riferimento ad un nuovo componente di tipo <code>METALabel</code> contenente la stringa specificata
	 * e delle dimensioni indicate.
	 */
    public ILabel createLabel(String s, Dimension d)
    {
	return new METALabel(d.width, d.height, s, labelbgColor, textColor, textFont, d.height / 2);
    }
	
	/**
	 * Restituisce il riferimento ad un nuovo componente di tipo <code>METAText</code> contenente la stringa specificata
	 * e delle dimensioni indicate.
	 */
    public IText createText(String s, Dimension d)
    {
	return new METAText(d.width, d.height, s, textbgColor, textColor, textFont, d.height / 2);
    }
	
	/**
	 * Restituisce il riferimento ad un nuovo componente di tipo <code>METABooleanButton</code> delle dimensioni specificate e
	 * con lo stato iniziale di default.
	 */
    public IBooleanButton createBooleanButton(boolean value, Dimension d)
    {
	return new METABooleanButton(d.width, d.height, booleanbuttonValue, textbgColor, textColor);
    }
}

