/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
***********************************************************************/

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * Un oggetto appartenente alla classe <code>METAContainer</code>  un componente contenitore
 * per altri sotto-componenti: a lui  affidato il compito di propagare gli eventi verso tutti i sotto-componenti
 * che contiene ed  incaricato di ridisegnarli ogni qual volta sia necessario.
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto.
 */
public class METAContainer extends METAComponent implements IContainer
{
	/** ArrayList contenente tutti i sotto-componenti. */
    private ArrayList items = new ArrayList();

	/** Struttura che associa ad ogni sotto-componente del <code>METAContainer</code> le proprie coordinate. */
    private class Obj
    {
	
	/** Campi contenenti le coordinate del sotto-componente. */
	public int x, y;

	/** Campo contenente il riferimento al sotto-componente. */
	public IComponent comp;
	
	/**Crea un nuovo oggetto <code>Obj</code> che contiene il sotto-componente specificato alle coordinate specificate. */
	public Obj(int x, int y, IComponent c)
	{
	    this.x = x;
	    this.y = y;
	    this.comp = c;
	}
    }
	
	/** Aggiunge il sotto-componente alle coordinate specificate all'interno del <code>METAContainer</code>. */
    public void add(int x, int y, IComponent c)
    {
	items.add(new Obj(x, y, c));
    }
	
	/** Rimuove il sotto-componente indicato dal <code>METAContainer</code>. */
    public void remove(IComponent c)
    {
	Iterator foo = items.iterator();

	while(foo.hasNext())
	    {
		if(((Obj)foo.next()).comp.equals(c))
		    items.remove(items.indexOf(foo.next()));
	    }
    }

	/** Si occpura di disegnare tutti i sotto-componenti contenuti nell'oggetto <code>Graphics</code> specificato. */
    public void paint(Graphics g)
    {
	Iterator bar = items.iterator();

	while(bar.hasNext())
	    {
		Obj next = (Obj)bar.next();
		Dimension dim = next.comp.getSize();
		Graphics graph = g.create(next.x, next.y, dim.width, dim.height); 
		next.comp.paint(graph);
	    }
    }
	
	/**
	 * Restituisce un riferimento al sotto-componente contenuto nel <code>METAContainer</code> che si trova alle
	 * coodrinate specificate.
	 */
    public IComponent getComponent(int x, int y)
    {
	Iterator man = items.iterator();
    
	while(man.hasNext())
	    {
		Obj next = (Obj)man.next();
		if(next != null  &&  next.x == x  &&  next.y == y)
		    return (IComponent)next.comp;
	    }

	return null;
    }
	
	/** Restituisce la dimensione del componente: per definizione in container non viene considerato in termini dimensionali. */
    public Dimension getSize()
    {
	return null;
    }

	/**
	 * Si occupa di propagare verso tutti i sotto-componenti contenuti l'evento individuato dalle coordinare del mouse
	 * a seguito di un click, selezionando il sotto-componente associato e verificando se tale sotto-componente  in grado
	 * o meno di gestire e quindi consumare l'evento; in tal caso restituisce true, in caso contratio false.
	 */
    public boolean handle(IEvent ev)
    {
	String className = ev.getClass().getName();
	if(className.equals("METAMousePressedEvent") || className.equals("METAMouseReleasedEvent"))
	    {
		int x, y;
		Iterator baz = items.iterator();
		
		if(className.equals("METAMousePressedEvent"))
		    {
			x = ((METAMousePressedEvent)ev).getPoint().x;
			y = ((METAMousePressedEvent)ev).getPoint().y;
		    }
		else 
		    {
			x = ((METAMouseReleasedEvent)ev).getPoint().x;
			y = ((METAMouseReleasedEvent)ev).getPoint().y;
		    }
		
		while(baz.hasNext())
		    {
			Obj next = (Obj)baz.next();
			
			if(next != null)
			    {
				Dimension dim = next.comp.getSize();
				if(x >=  next.x  &&  x <= next.x + dim.width  &&  y >= next.y  &&  y <= next.y + dim.height)
				    {
					if(className.equals("METAMousePressedEvent"))	    
					    return next.comp.handle((METAMousePressedEvent)ev);
					else 
					    return next.comp.handle((METAMouseReleasedEvent)ev);
				    }
			    }
		    }
	    }
	else if(className.equals("METAKeyPressedEvent") || className.equals("METAKeyReleasedEvent"))
	    {
		Iterator doge = items.iterator();
		while(doge.hasNext())
		    {
			Obj next = (Obj)doge.next();
			
			if(next != null)
			    {
				if(next.comp.getClass().getName().equals("METAButton"))
				    {
					if(className.equals("METAKeyPressedEvent"))
					    next.comp.handle((METAKeyPressedEvent)ev);
					else
					    next.comp.handle((METAKeyReleasedEvent)ev);
				    }
			    }
		    }
		return true;
	    }
	return false;
    }
}

