/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
***********************************************************************/

import java.util.*;
import java.awt.*;

/**
 * Un oggetto appartenente alla classe <code>METALabel</code>  un'etichetta di testo che pu essere
 * inserita all'interno del container <code>METAContainer</code> associato ad una finestra <code>METAWIndow</code>.
 *
 * @see METAContainer
 * @see METAWindow
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto
 */
public class METALabel extends METAComponent implements ILabel
{
	/** Campo testo dell'etichetta. */
    private String label;

	/** Colore di sfondo dell'etichetta. */
    private Color bgColor;

	/** Colore del testo dell'etichetta. */
    private Color textColor;

	/** Font del testo dell'etichetta. */
    private Font font;

	/** Raggio di curvatura associato al livello di arrotondamento degli angoli della cella rettangolare contenente il testo. */
    private int ray;
	
	/** Crea un nuovo oggetto di tipo <code>METALabel</code> con le caratteristiche iniziali specificate dai parametri.
	 *
	 * @param w larghezza dell'etichetta di testo
	 * @param h altezza dell'etichetta di testo
	 * @param s stringa contenente il testo iniziale dell'etichetta
	 * @param bg colore di sfondo dell'etichetta
	 * @param text colore del testo
	 * @param font font del testo
	 * @param r raggio di curvatura del raccordo sugli angoli della cella contenente il testo
	 */
    public METALabel(int w, int h, String s,  Color bg, Color text, Font f, int r)
    {
	this.width = w;
	this.height = h;
	bgColor = bg;
	textColor = text;
	font = f;
	label = s;
	ray = r;
    }
	
	/** Restituisce una stringa contenente il testo dell'etichetta. */
    public String getLabel()
    {
	return label;
    }
	
	/** 
	 * Modifica il testo contenuto nell'etichetta sostituendolo con la stringa fornita aggiornando al contempo
	 * la visualizzazione.
	 */
    public void  setLabel(String s)
    {
	label = s;
    }
	
	/**
	 * Si occupa di disegnare l'etichetta di testo nell'oggetto <code>Graphics</code> specificato.
	 *
	 * @param g oggetto graphics in cui disegnare l'etichetta di testo
	 */
    public void paint(Graphics g)
    {
	this.graphics = g;
	g.setColor(bgColor);
	g.drawRoundRect(0, 0, this.width, this.height, ray, ray);
	g.fillRoundRect(0, 0, this.width, this.height, ray, ray);
	g.setColor(textColor);
	g.setFont(font);
	FontMetrics fontM = g.getFontMetrics();
	g.drawString(label, (this.width - fontM.stringWidth(label)), this.height / 2 + fontM.getAscent() / 2);
    }

    /**
	 * Permette di verificare se il componente  in grado di gestire l'evento passato e restituisce <code>true</code> nel
	 * caso in qui l'evento sia consumato, <code>false</code> in caso contrario.
	 */
	public boolean handle(IEvent e)
    {
	if(this.getListener() != null)
	    {	
		this.getListener().handle(this, e);
		paint(this.graphics);
		return true;
	    }
	return false;
    }
}

