/***********************************************************************
This file is part of DogeLibs.
Dogelibs: a library of simple graphical widgets

Copyright (C) 2003  Roberto Campesato, Andrea Sottoriva

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA
***********************************************************************/

import java.util.*;
import java.awt.*;

/**
 * Un oggeto appartenente alla classe <code>METAText</code>  un'area di testo editabile
 * attraverso eventi di pressione di tasti sulla tastiera.
 *
 * @version 1.0
 * @author Sottoriva Andrea, Campesato Roberto
 */
public class METAText extends METAComponent implements IText
{
	
	/** Stringa di testo contenuta nell'area editabile. */
    private String text = null;

	/** Colore di sfondo dell'area di testo editabile. */
    private Color bgColor;

	/** Colore del testo contenuto nell'area di testo editabile. */
    private Color textColor;

	/** Font del testo contenuto nell'area di testo editabile. */
    private Font font;

	/** Raggio di curvatura del raccordo sugli angoli del rettangolo rappresentante l'area di testo editabile. */
    private int ray;

	/** 
	 * Crea un oggetto di tipo <code>METAText</code> con le caratteristiche iniziali specificate dai parametri
	 * illustrati in seguito.
	 *
	 * @param w larghezza dell'area di testo editabile
	 * @param h altezza dell'area di testo editabile
	 * @param s stringa iniziale contenuta nell'area di testo
	 * @param bg colore di stondo dell'area di testo
	 * @param text colore del testo contenuto nell'area
	 * @param font font del testo
	 * @param r raggio di arrotondamento degli anngoli del rattangolo rappresentante l'area di testo editabile
	 */
    public METAText(int w, int h, String s, Color bg, Color text, Font f, int r)
    {
	this.width = w;
	this.height = h;
	this.text = s;
	bgColor = bg;
	textColor = text;
	font = f;
	ray = r;
    }

	/** Restituisce la stringa di testo contenuta nell'area editabile. */
    public String getText()
    {
	return text;
    }
	
	/** Altera il contenuto dell'area di testo editabile sostituendolo con la stringa fornita. */
    public void  setText(String s)
    {
	text = s;
    }

	/**
	 * Si occupa di disegnare l'area di testo editabile all'interno dell'oggetto <code>Graphics</code> specificato
	 * @param g oggetto Graphics in cui disegnare l'area di testo editabile.
	 */
    public void paint(Graphics g)
    {
	this.graphics = g;
	g.setColor(bgColor);
	g.drawRoundRect(0, 0, this.width, this.height, ray, ray);
	g.fillRoundRect(0, 0, this.width, this.height, ray, ray);
	g.setColor(textColor);
	g.setFont(font);
	if(text != null)
	    {
		FontMetrics fontM = g.getFontMetrics();
		g.drawString(text, (this.width - fontM.stringWidth(text)), this.height / 2 + fontM.getAscent() / 2);
		}
    }

    /**
	 * Si occupa di verificare se il componente  in grado di gestire l'evento passato e restituisce
	 * true nel caso in cui l'evento sia stato consumato, false in caso contrario.
	 */
	public boolean handle(IEvent e)
    {
	if(e.getClass().getName().equals("METAKeyReleasedEvent")) 
	    {
		if(this.getListener() != null) 
		    {
			this.getListener().handle(this, e);
			paint(this.graphics);
			return true;
		    }
		else return false;
	    }
	else return false;
    }
}

